import heapq

def solve():
    n, m, k = map(int, input().split())
    edges = []
    for _ in range(m):
        u, v = map(int, input().split())
        # More efficient to store edges as an adjacency list
        # Improves performance significantly for larger graphs
        edges.append((u - 1, v - 1))

    guards = []
    for _ in range(k):
        p, h = map(int, input().split())
        guards.append((p - 1, h))

    def dijkstra(start_node, max_dist):
        # Use an adjacency list for faster neighbor lookup
        adj = [[] for _ in range(n)]
        for u, v in edges:
            adj[u].append(v)

        distances = {i: float('inf') for i in range(n)}
        distances[start_node] = 0
        priority_queue = [(0, start_node)]
        reachable = set()

        while priority_queue:
            dist, current_node = heapq.heappop(priority_queue)

            if dist > max_dist:
                continue

            reachable.add(current_node)  # Add to reachable set immediately

            for neighbor in adj[current_node]:
                if distances[neighbor] > dist + 1:
                    distances[neighbor] = dist + 1
                    heapq.heappush(priority_queue, (distances[neighbor], neighbor))

        return reachable

    protected_warehouses = set()
    for guard_pos, guard_range in guards:
        protected_warehouses.update(dijkstra(guard_pos, guard_range))

    protected_warehouses_list = sorted(list(protected_warehouses))
    print(len(protected_warehouses_list))
    if len(protected_warehouses_list) > 0:
        print(*(x + 1 for x in protected_warehouses_list))

solve()